--- arr4
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.arr4

local historie = [[
	Version 1.1:
	 - Release
]]

------------------------------------------------------------------------------------
--	Aktuelle Versionsdaten.
------------------------------------------------------------------------------------
local versionsData = {
	majorVersion = 1,
	minorVersion = 1,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

local vec4
------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		vec4 = global.vec4
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		for modul in moduleData:_pairs() do
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local function getArr4(x, y, z, w)
	if (type(x) == "table") then
		if ((type(x[1]) == "number") and (type(x[2]) == "number")) then
			-- x is a arr2 or arr3
			return {x[1], x[2], (type(x[3]) == "number") and x[3] or 0, (type(x[4]) == "number") and x[4] or 0}
		elseif ((type(x.x) == "number") and (type(x.y) == "number")) then
			-- x is a vec2, vec3 or vec4
			return {x.x, x.y, (type(x.z) == "number") and x.z or 0, (type(x.w) == "number") and x.w or 0}
		end
	elseif (type(x) == "number") then
		-- x is a number; We look to see if y, z and w are also numbers. Otherwise they get x as value
		return {x, (type(y) == "number") and y or x, (type(z) == "number") and z or x, (type(w) == "number") and w or x}
	end
	-- nothing; return default
	return {0, 0, 0, 0}
end

local mt
mt = {
  __add = function (self, rhs) -- "add" event handler
		local v = getArr4(rhs)
		return setmetatable({self[1] + v[1], self[2] + v[2], self[3] + v[3], self[4] + v[4]}, mt)
  end,
	__sub = function (self, rhs) -- "sub" event handler
		local v = getArr4(rhs)
		return setmetatable({self[1] - v[1], self[2] - v[2], self[3] - v[3], self[4] - v[4]}, mt)
  end,
	__mul = function (self, rhs) -- "mul" event handler
		local v = getArr4(rhs)
		return setmetatable({self[1] * v[1], self[2] * v[2], self[3] * v[3], self[4] * v[4]}, mt)
	end,
	__div = function (self, rhs) -- "div" event handler
		local v = getArr4(rhs)
		return setmetatable({self[1] / v[1], self[2] / v[2], self[3] / v[3], self[4] / v[4]}, mt)
	end,
	__index = {
		toParams = function(self)
			return self[1], self[2], self[3], self[4]
		end,
		toVec = function(self)
			return vec4.new(self)
		end,
		toString = function(self)
			return tostring(self)
		end,
	},
	__tostring = function(self)
		return string.format("{%.4f, %.4f, %.4f, %.4f}", self:toParams())
	end,
}
------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--	arr4.new(x, y, z, w):
	--	1.	x is a number. Then y, z and w can also be numbers. If not, they take the value of x.
	--	2.	x is a point (arr3)
	--	3.	x is an arr2, then z becomes 0
	--	4.	x is a vec3 / vec4
	--	5.	x is an vec2, then z becomes 0
	--	Otherwise returns vec3 with {x = 0, y = 0, z = 0}
	------------------------------------------------------------------------------------
	new = function(x, y, z, w)
		return setmetatable(getArr4(x, y, z, w), mt)
	end,
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local modulDB = setmetatable({}, {
	__index = function(self, key)
		if modules then
			for modul in modules:_pairs() do
				local value = modul.handler[key]
				if (value ~= nil) then
					return value
				end
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})

return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass, modulDB}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
