--- codeAdder
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.codeAdder

local historie = [[
	Version 1.1:
	 - Release
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 1,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

local string
local table

------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		string = global.string
		table = global.table
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		for modul in moduleData:_pairs() do
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--	createCodeAdder
	--	
	------------------------------------------------------------------------------------
	create = function(sOpen, sClose, addEntrySeperator)
		local sReturn = "\n"
		local sSpace = "\t"
		local sSep = ","
		
		local function getAdder(parent, sOpen, sClose, addEntrySeperator)
			local mt_adder = {
				__tostring = function(self)
					local stack = {}
					table.insert(stack, self.sOpen)
					
					for value in self.content:_pairs() do
						table.insert(stack, ((type(value) == "string") and value or tostring(value)))
					end
					table.insert(stack, self.sClose)
					
					return table.concat(stack)
				end,
				__index = {
					addBlock = function(self, sOpen, sClose, addEntrySeperator)
						return getAdder(self, sOpen, sClose, addEntrySeperator)
					end,
					addCode = function(self, sCode, addEntrySeperator)
						addEntrySeperator = (type(addEntrySeperator) ~= "boolean") and true or addEntrySeperator
						local sEnd = (addEntrySeperator and sSep..sReturn or sReturn)
						for value in string.split(sCode, sReturn):_pairs() do
							self.content:insert(self.sSpace..value..sEnd)
						end
					end,
					addContent = function(self, sContent, addEntrySeperator)
						addEntrySeperator = (type(addEntrySeperator) ~= "boolean") and true or addEntrySeperator
						self.content:insert(addEntrySeperator and self.sSpace..sContent..sSep..sReturn or self.sSpace..sContent..sReturn)
					end,
					addFunction = function(self, name, params, addEntrySeperator)
						return getAdder(self, string.format("%s = function(%s)", name, table.concatTable(params, sSep.." ")), "end", addEntrySeperator)
					end,
					addKey = function(self, key, params, addEntrySeperator)
						local sKey = (type(key) == "string") and key.." = " or ""
						self:addContent(sKey..eatToString(params, self.sSpace), addEntrySeperator)
					end,
					toFile = function(self, fileName)
						local file = io.open(fileName, "w")
						if file then
							file:write(self:tostring())
							file:close()
						end
					end,
					print = function(self)
						print(self:tostring())
					end,
					tostring = function(self)
						return tostring(self)
					end,
				},
			}
			
			local sParentSpace = ((type(parent) == "table") and (type(parent.sSpace) == "string")) and parent.sSpace or ""
			addEntrySeperator = (type(addEntrySeperator) ~= "boolean") and true or addEntrySeperator
			
			local result = {
				sSpace = sParentSpace..sSpace,
				sOpen = sParentSpace..sOpen..sReturn,
				sClose = addEntrySeperator and sParentSpace..sClose..sSep..sReturn or sParentSpace..sClose..sReturn,
				content = eatSetContainer(),
			}
			
			if ((type(parent) == "table") and (type(parent.content) == "table")) then
				parent.content:insert(result)
			end
			
			return setmetatable(result, mt_adder)
		end
		
		return getAdder(nil, sOpen, sClose, (type(addEntrySeperator) ~= "boolean") and false or addEntrySeperator)
	end,
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local modulDB = setmetatable({}, {
	__index = function(self, key)
		if modules then
			for modul in modules:_pairs() do
				local value = modul.handler[key]
				if (value ~= nil) then
					return value
				end
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})

return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass, modulDB}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
