--- modutils
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.modutils

local historie = [[
	Version 1.1:
	 - Release
	Version 1.7:
	 - Added: Behandlung weiterer Usereinstellungen hinzugefügt
	Version 1.8:
	 - interne Version
	Version 1.9:
	 - Changed: komplette Umstellung von "settings.lua"-Unterstützung auf interne Game-UI
	 - Fixed: Versionskonflikte in function init() behoben. Daten älterer Versionen wurden gelöscht. Ab jetzt jede Version mit eigenen Daten.
	Version 1.10:
	 - Added: Benutzerdefinierte Tabellenänderungen (userSelection.tablesToChange).
	Version 1.11:
	 - Changed: Verwendung von eatSetEx anstatt eatset
	Version 1.12:
	 - Changed: Switch to new system of initialization eatglopal-API (postLoadFn).
	 - Changed: Implementation of a modular system.
	Version 1.13:
	 - Changed: Einzelne Dateien in modelFiles wieder als String anstatt Tabelle möglich.
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 13,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

local misc
local table
local getValue
local isValidId

------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		misc = global.misc
		table = global.table
		getValue = misc.getValue
		isValidId = misc.isValidId
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		logFn("registerModule: moduleData = ", moduleData)
		for modul in moduleData:_pairs() do
			logFn("registerModule: modul.metaData.name = ", modul.metaData.name)
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
--	Tool(s)
------------------------------------------------------------------------------------
local function init()
	-- globale Daten
	if (type(eat1963_global_data) ~= "table") then
		eat1963_global_data = {}
	end
	---	modutils
	--		Ab Version 2.30 ändert sich der Schlüssel von "modutils" auf "modutils_v2_1",
	--		da sich modelHandler(fileName, data, key) geändert hat!!!
	if (type(eat1963_global_data.modutils_v2_1) ~= "table") then
		eat1963_global_data.modutils_v2_1 = {
			modelHandler = {
				modelHandlerRegistered = false,
				models = {},
			},
			modData = {},
		}
	end
end

local function getGlobalModUtils()
	return eat1963_global_data.modutils_v2_1
end

------------------------------------------------------------------------------------
--	getAsSet
------------------------------------------------------------------------------------
local function getAsSet(value)
	if value then
		return (type(value) ~= "table") and eatSetContainer({value}) or eatSetContainer(value)
	end
end

------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--	Tabelle userSettings
	--	
	------------------------------------------------------------------------------------
	userSettings = {
		create = function(modData)
			------------------------------------------------------------------------------------
			--	HIER geth's los. :)
			------------------------------------------------------------------------------------
			init()
			local globalModUtils = getGlobalModUtils()
			local modId = getCurrentModId()
			------------------------------------------------------------------------------------
			--	Tabelle result
			------------------------------------------------------------------------------------
			local result = {
				------------------------------------------------------------------------------------
				--	execute
				--	
				------------------------------------------------------------------------------------
				execute = function(modParams)
					local modData = globalModUtils.modData[modId]
					local intern = modData.modutilsIntern
					------------------------------------------------------------------------------------
					--	params für späteren Zugriff sichern
					--		Speichert modParams des aktuellen Mods global ab.
					--		Alle Werte werden auf Gültigkeit geprüft und um 1 erhöht.
					--		Somit sind diese bereits fit für evtl. lua-Tabellenauswahl per Index. :)
					------------------------------------------------------------------------------------
					if (type(modParams) == "table") then
						local params
						params = modParams[modId]
						if (type(params) ~= "table") then
							params = modParams
						end
						--local params = getValue(modParams[getCurrentModId()], {})
						for value in modData.myParams.params:_pairs() do
							params[value.key] = getValue(params[value.key], getValue(value.defaultIndex, 0)) + 1
						end
						intern.userParams = params
					end
					------------------------------------------------------------------------------------
					--	Dateien aus Tabelle "intern.hideFiles" ausblenden.
					------------------------------------------------------------------------------------
					global.mod.hideObject(intern.hideFiles)
					------------------------------------------------------------------------------------
					--	Module ausführen
					------------------------------------------------------------------------------------
					if modules then
						for modul in modules:_pairs() do
							if (type(modul.handler.run) == "function") then
								logFn("run ", modul.metaData.name, ".run. modData = ", modData)
								modul.handler.run(modData)
							end
						end
					end
				end,
			}
			
			if (type(globalModUtils.modData[modId]) ~= "table") then
				------------------------------------------------------------------------------------
				--	Kopie von modData erstellen und global abspeichern
				------------------------------------------------------------------------------------
				globalModUtils.modData[modId] = table.copyTable(getValue(modData, {}))
				------------------------------------------------------------------------------------
				--	Hilfsvariable mData für einfacheren Zugriff anlegen.
				------------------------------------------------------------------------------------
				local mData = globalModUtils.modData[modId]
				--	Daten-Validität checken/herstellen
				mData.general = getValue(mData.general, {})
				mData.userSelection = getValue(mData.userSelection, {})
				
				--[[
				for autoFile in eatSetEx.new(getValue(modData.userSelection.autoFiles, {})):_pairs() do
					for fileData in eatSetEx.new(getValue(modData.userSelection.files, {})):_pairs() do
						fileData.autoFiles = {}
						local autoFiles = eatSetEx.new()
						for file in eatSetEx.new(getValue(modData.userSelection.files.modelFiles, {})):_pairs() do
							autoFiles:insert(utils.fileNameIncludeSuffix(file, autoFile.autoString))
						end
						fileData.autoFiles[autoFile.key] = autoFiles
					end
				end
				]]
				------------------------------------------------------------------------------------
				--	Interne Datenstruktur anlegen.
				------------------------------------------------------------------------------------
				mData.modutilsIntern = {
					availableModelFiles = eatSetEx.new(),
					generalYearFiles = eatSetEx.new(),
					moduleData = {},
					
					autoFiles = {},
					hideFiles = eatSetEx.new(),
					userParams = {},
					dataTablesToChange = eatSetEx.new(),
				}
				------------------------------------------------------------------------------------
				--	Interne Datenstruktur initialisieren.
				------------------------------------------------------------------------------------
				--	availableModelFiles füllen
				local userSelection = getValue(mData.userSelection, {})
				for fData in eatSetContainer(userSelection.files):_pairs() do
					mData.modutilsIntern.availableModelFiles:append(getValue(getAsSet(fData.modelFiles), {}))
				end
				
				--	Tabelle "modData.general" behandeln
				local general = getValue(mData.general, {})
				global.mod.hideObject(general.hideFiles, true)
				mData.modutilsIntern.generalYearFiles:append(getValue(general.yearFiles, {}))
				
				------------------------------------------------------------------------------------
				--	Tool für UI erzeugen
				------------------------------------------------------------------------------------
				mData.myParams = global.paramutils.create()
				
				------------------------------------------------------------------------------------
				--	createUI der Module aufrufen
				------------------------------------------------------------------------------------
				logFn("modules = ", modules)
				if modules then
					for modul in modules:_pairs() do
						logFn("createUI: modul.name = ", modul.name)
						if (type(modul.handler.createUI) == "function") then
							logFn("run ", modul.metaData.name, ".createUI")
							modul.handler.createUI(mData)
						end
					end
				end
				
				--[[
				-- UI erstellen
				createUI(globalModUtils.modData[modId].myParams, modData)
				]]
			end

			result.myParams = globalModUtils.modData[modId].myParams
			result.getUserSelections = function() return globalModUtils.modData[modId].modutilsIntern.userParams end
			return result, true, ""
		end,
	},
}

return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
