--- repository
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.repository

local historie = [[
	Version 1.1:
	 - Release
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 1,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback

------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--	createRep
	--	
	------------------------------------------------------------------------------------
	create = function(_api)
		local mt_data = {
			__index = function(self, key)
				local result = {
					name = key,
				}
				local val = rawget(self, key)
				if (val ~= nil) then
					result.data = val
				else
					local index = _api.find(key)
					if (index > -1) then
						val = _api.get(index)
						if (val ~= nil) then
							rawset(self, key, val)
							result.data = val
						end
					end
				end
				return result
			end,
		}
		local dataClass =  setmetatable({}, mt_data)
		
		local mt = global.table.copyTable(getmetatable(eatSetEx))
		mt.__newindex = function(self, key, value)
		end
		mt.__pairs = function(self)
			local i = 0
			return function(self, key)
				i = i + 1
				local entry = self[i]
				if global.misc.isValidId(entry) then
					return entry, dataClass[entry].data
				end
			end, self, nil
		end
		mt.__ipairs = function(self)
			return function(self, i)
				i = i + 1
				local entry = self[i]
				if global.misc.isValidId(entry) then
					return i, dataClass[entry].data
				end
			end, self, 0
		end
		mt.__index.getEntry = function(self, name)
			logFn("name = ", name)
			return dataClass[name].data
		end
		mt.__index._pairs = function(self, orderFnc, filterFnc)
			-- collect the keys
			local keys = {}
			for i = 1, #self do
				keys[#keys + 1] = i
			end
			
			if orderFnc then
				table.sort(keys, function(a, b) return orderFnc(dataClass[self[a]], dataClass[self[b]]) end)
			end
			
			local doFilter = buildFilterFnc(filterFnc)
			-- return the iterator function
			local i = 0
			return function()
				i = i + 1
				while keys[i] and (not doFilter(dataClass[self[keys[i]]])) do
					i = i + 1
				end
				if keys[i] then
					return dataClass[self[keys[i]]]
				end
			end
		end
		
		mt.__index.count = function(self, filterFnc)
			local result = 0
			local doFilter = buildFilterFnc(filterFnc)
			for i = 1, #self do
				if doFilter(dataClass[self[i]]) then
					result = result + 1
				end
			end
			return result
		end
		
		local result = {}
		for __, name in pairs(_api.getAll()) do
			table.insert(result, name)
		end
		return setmetatable(result, mt)
	end,
}

local mt = {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
}

return setmetatable({}, mt)