--- Serialize
-- @author Enno Sylvester
-- @copyright 2017, 2019, 2021, 2023
-- @module eatglobal.serialize

local historie = [[
	Version 1.6:
	 - Changed: Einführung Historie
	 - Changed: Alle Functionen nutzen nun globale function eatToString.
	Version 1.7:
	 - Added: Modulfähigkeit (Erweiterungsmöglichkeiten durch Module) implementiert.
	 - Changed: Switch to new system of initialization eatglopal-API (postLoadFn).
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 7,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		for modul in moduleData:_pairs() do
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--- Serialisiert 'object' und gibt die "reinen" Daten zurück
	--	Format:	{
	--						value1, value2, etc.
	--					}
	-- @libResult.serializeBlanc
	-- @param object
	-- @return String
	------------------------------------------------------------------------------------
	serializeBlanc = function(object)
		return eatToString(object)
	end,

	------------------------------------------------------------------------------------
	--- Serialisiert 'object' und gibt Data-Format zurück
	--	Format:	function data()
	--						return {
	--							value1, value2, etc.
	--						}
	--					end
	-- @libResult.serializeDatafmt
	-- @param object
	-- @return String
	------------------------------------------------------------------------------------
	serializeDatafmt = function(object)
		return "function data()\nreturn "..eatToString(object).."\nend\n"
	end,

	------------------------------------------------------------------------------------
	--- Serialisiert 'object' und gibt Return-Format zurück
	--	Format:	return {
	--						value1, value2, etc.
	--					}
	-- @libResult.serializeReturnfmt
	-- @param object
	-- @return String
	------------------------------------------------------------------------------------
	serializeReturnfmt = function(object)
		return "return "..eatToString(object).."\n"
	end,

	------------------------------------------------------------------------------------
	--- Serialisiert 'object' und gibt Table-Format zurück
	--	Format:	local nameOfTable or "table" =
	--						{
	--							value1, value2, etc.
	--						}
	-- @libResult.serializeTablefmt
	-- @param object
	-- @param nameOfTable
	-- @return String
	------------------------------------------------------------------------------------
	serializeTablefmt = function(object, nameOfTable)
		return "local "..(nameOfTable or "table").." = "..eatToString(object).."\n"
	end,

	------------------------------------------------------------------------------------
	--- Serialisiert 'object' und gibt Table/Return-Format zurück
	--	Format:	local nameOfTable or "table" =
	--						{
	--							value1, value2, etc.
	--						}
	--
	--						return nameOfTable or "table"
	-- @libResult.serializeTablefmt_Returnfmt
	-- @param object
	-- @param nameOfTable
	-- @return String
	------------------------------------------------------------------------------------
	serializeTablefmt_Returnfmt = function(object, nameOfTable)
		return "local "..(nameOfTable or "table").." = "..eatToString(object).."\n\nreturn "..(nameOfTable or "table")
	end,
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local modulDB = setmetatable({}, {
	__index = function(self, key)
		if modules then
			for modul in modules:_pairs() do
				local value = modul.handler[key]
				if (value ~= nil) then
					return value
				end
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})

return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass, modulDB}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
