--- string
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.string

local historie = [[
	Version 1.1:
	 - Release
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 1,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		for modul in moduleData:_pairs() do
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
local splitString

functionClass = {
	------------------------------------------------------------------------------------
	--	split
	--	Sucht in 's' nach 'delimiter' und gibt die Teilstücke in einer Liste zurück.
	--	Ist 'plain' == true oder nil, werden keine Pattern, wie z.B. "/n" o.ä. in 's'
	--	berücksichtigt.
	--	Mit einer "filterFn" kann die Ausgabe gefiltert werden. Z.B. werden mit
	--	"function(tail) return global.misc.isValidId(tail) end" keine "" zurückgegeben.
	--
	--	Beispiele:
	--	s																	delimiter						Ergebnis
	--	"mods/eat1963_streets/scripts"		"/"									{"mods", "eat1963_streets", "scripts"}
	--	"mods/eat1963_streets/scripts/"		"/"									{"mods", "eat1963_streets", "scripts", ""}
	--	"mods/eat1963_streets/scripts"		"eat1963_streets"		{"mods/", "/scripts/"}
	--	"mods/eat1963_streets/scripts"		"mods"							{"", "/eat1963_streets/scripts"}
	--	"mods/eat1963_streets/scripts"		"s/"								{"mod", "eat1963_street", "scripts"}
	--	"1.14"														"."									{"1", "4"}
	--	".1.14."													"."									{"", "1", "4", ""}
	--
	--	Rückgabe als eatSetContainer.
	------------------------------------------------------------------------------------
	split = function(s, delimiter, plain, filterFn)
		if (splitString == nil) then
			splitString = getVar("eat.intern.splitString")
		end
		return eatSetContainer(splitString(s, delimiter, plain, filterFn))
	end,
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local modulDB = setmetatable({}, {
	__index = function(self, key)
		if modules then
			for modul in modules:_pairs() do
				local value = modul.handler[key]
				if (value ~= nil) then
					return value
				end
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})

return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass, modulDB, string}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
