--- transf
-- @author Enno Sylvester
-- @copyright 2018, 2019, 2021, 2023
-- @module eatglobal.transf

local transf = require "transf"

local historie = [[
	Version 1.4:
	 - Changed: Einführung Historie
	 - Changed: libResult.create, setRotation, setScale, scaleRotXYZTransl - Result-libResult gerundet auf max. 3 Stellen nach Komma und Werte +/- 0.001 gerundet auf 0
	Version 1.5:
	 - Added: Modulfähigkeit (Erweiterungsmöglichkeiten durch Module) implementiert.
	 - Changed: Switch to new system of initialization eatglopal-API (postLoadFn).
	 - Changed: Original-transf wird nun nicht mehr kopiert. Zugriff kombiniert auf lokale, neue Funktionen UND originale transf. Steuerung per Metatable.
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 5,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

local arr3
local math
local vec3
------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		arr3 = global.arr3
		math = global.math
		vec3 = global.vec3
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		for modul in moduleData:_pairs() do
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--	copy
	------------------------------------------------------------------------------------
	copy = function(_transf)
		local result = {}
		for i = 1, #_transf do
			result[i] = _transf[i]
		end
		
		return result
	end,
	------------------------------------------------------------------------------------
	--	create
	------------------------------------------------------------------------------------
	create = function(position, rotation, scale, rotationIsRad)
		local rot = arr3.new(rotation)
		rotationIsRad = (type(rotationIsRad) == "boolean") and rotationIsRad or false
		if rotationIsRad then
			return math.clearNumberArray(functionClass.scaleRotZYXTransl(vec3.new(scale or {1, 1, 1}),
				vec3.new(rot):swap(), vec3.new(position)))
		else
			return math.clearNumberArray(functionClass.scaleRotZYXTransl(vec3.new(scale or {1, 1, 1}),
				functionClass.degToRad(rot[3], rot[2], rot[1]),
					vec3.new(position)))
		end
	end,
	------------------------------------------------------------------------------------
	--	getPos
	------------------------------------------------------------------------------------
	getPos = function(_transf, pos)
		return arr3.new({
			_transf[13],
			_transf[14],
			_transf[15],
		})
	end,
	------------------------------------------------------------------------------------
	--	setPos
	------------------------------------------------------------------------------------
	setPos = function(_transf, pos)
		local result = functionClass.copy(_transf)
		result[13] = pos[1]
		result[14] = pos[2]
		result[15] = pos[3]
		
		return result
	end,
	------------------------------------------------------------------------------------
	--	incrementPos
	------------------------------------------------------------------------------------
	incrementPos = function(_transf, increment)
		local result = functionClass.copy(_transf)
		result[13] = result[13] + increment[1]
		result[14] = result[14] + increment[2]
		result[15] = result[15] + increment[3]
		
		return result
	end,
	------------------------------------------------------------------------------------
	--	setRotation
	------------------------------------------------------------------------------------
	setRotation = function(_transf, rotation, origin, rotationIsRad)
		--[[
			Rotationen immer um den Nullpunkt, sonst landet das Objekt irgendwo.
			Daher:
				1. Position merken
				2. Position auf 0
				3. rotieren
				4. gemerkte Position wieder herstellen
		]]
		local pos = functionClass.getPos(_transf)
		origin = origin or {0, 0, 0}
		rotationIsRad = (type(rotationIsRad) == "boolean") and rotationIsRad or false
		if rotationIsRad then
			return math.clearNumberArray(functionClass.setPos(transf.mul(transf.rotZYXTransl({x = rotation[3], y = rotation[2], z = rotation[1]},
				vec3.new(0, 0, 0)), functionClass.setPos(_transf, origin)), pos))
		else
			return math.clearNumberArray(functionClass.setPos(transf.mul(transf.rotZYXTransl(transf.degToRad(rotation[3], rotation[2], rotation[1]),
				vec3.new(0, 0, 0)), functionClass.setPos(_transf, origin)), pos))
		end
	end,
	------------------------------------------------------------------------------------
	--	setScale
	------------------------------------------------------------------------------------
	setScale = function(_transf, scale)
		return math.clearNumberArray(transf.mul(_transf, transf.scale(vec3.new(scale))))
	end,
	------------------------------------------------------------------------------------
	--	transVec
	------------------------------------------------------------------------------------
	transVec = function(vec0, vec1, vec2, vec3)
		return transf.new(
			{ x = vec0.x, y = vec0.y , z = vec0.z, w = 0 },
			{	x = vec1.x, y = vec1.y , z = vec1.z, w = 0 },
			{	x = vec2.x, y = vec2.y , z = vec2.z, w = 0 },
			{	x = vec3.x, y = vec3.y , z = vec3.z, w = 1 }
		)
	end,
	------------------------------------------------------------------------------------
	--	rotXYZTransl
	------------------------------------------------------------------------------------
	rotXYZTransl = function(rot, transl)
		return transf.rotZYXTransl({ x = rot.z, y = rot.y, z = rot.x}, transl)
	end,
	------------------------------------------------------------------------------------
	--	scaleRotXYZTransl
	------------------------------------------------------------------------------------
	scaleRotXYZTransl = function(scale, rot, transl)
		return math.clearNumberArray(transf.mul(functionClass.rotXYZTransl(rot, transl), transf.scale(scale)))
	end,
	------------------------------------------------------------------------------------
	--	getTransformFromUnitXVec
	------------------------------------------------------------------------------------
	getTransformFromUnitXVec = function(p0, p1)
		local a = vec3.new(1, 0, 0)
		local b = vec3.new(0, 1, 0)
		
		local v = p1 - p0
		
		local lenV = vec3.distance(p0, p1)
		assert(lenV > .0)
		
		local transl = transf.transl(p0)
		
		local c1 = vec3.normalize(v)
		local c2 = vec3.cross(a, v)
		c2 = (vec3.length(c2) > .0) and vec3.normalize(c2) or vec3.normalize(vec3.cross(b, v))
		
		local c3 = vec3.normalize(vec3.cross(c2, a))
		
		local rot = functionClass.transVec(c1, c2, c3, vec3.new({0, 0, 0}))
		
		local scale = transf.scale(vec3.new(lenV, 1, 1))
		
		return transf.mul(transf.mul(transl, rot), scale)
	end,
	------------------------------------------------------------------------------------
	--	getTransformFromUnitXPt
	------------------------------------------------------------------------------------
	getTransformFromUnitXPt = function(p0, p1)
		return functionClass.getTransformFromUnitXVec(vec3.new(p0), vec3.new(p1))
	end,
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local modulDB = setmetatable({}, {
	__index = function(self, key)
		if modules then
			for modul in modules:_pairs() do
				local value = modul.handler[key]
				if (value ~= nil) then
					return value
				end
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})

return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass, modulDB, transf}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
