--- vec4
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.vec4

local historie = [[
	Version 1.1:
	 - Release
]]

------------------------------------------------------------------------------------
--	Aktuelle Versionsdaten.
------------------------------------------------------------------------------------
local versionsData = {
	majorVersion = 1,
	minorVersion = 1,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback
local modules

local arr4
------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		arr4 = global.arr4
	end,
	------------------------------------------------------------------------------------
	--	registerModule
	--	
	------------------------------------------------------------------------------------
	registerModule = function(moduleData)
		for modul in moduleData:_pairs() do
			if (type(modul.handler) == "table") then
				if (not modules) then
					modules = eatSetEx.new()
				end
				if modules:insertUnique(modul, function(m1, m2) return (m1.metaData.name == m2.metaData.name) end) then
					if (type(modul.init) == "function") then modul.init() end
				end
			end
		end
	end,
}

------------------------------------------------------------------------------------
--	Vordefinition der "functionClass".
------------------------------------------------------------------------------------
local functionClass

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local function getVec4(x, y, z, w)
	if (type(x) == "table") then
		if ((type(x[1]) == "number") and (type(x[2]) == "number")) then
			-- x is a arr2 or arr3
			return {x = x[1], y = x[2], z = (type(x[3]) == "number") and x[3] or 0, w = (type(x[4]) == "number") and x[4] or 0}
		elseif ((type(x.x) == "number") and (type(x.y) == "number")) then
			-- x is a vec2, vec3 or vec4
			return {x = x.x, y = x.y, z = (type(x.z) == "number") and x.z or 0, w = (type(x.w) == "number") and x.w or 0}
		end
	elseif (type(x) == "number") then
		-- x is a number; We look to see if y, z and w are also numbers. Otherwise they get x as value
		return {x = x, y = (type(y) == "number") and y or x, z = (type(z) == "number") and z or x, w = (type(w) == "number") and w or x}
	end
	-- nothing; return default
	return {x = 0, y = 0, z = 0, w = 0}
end

local mt
mt = {
  __add = function (self, rhs) -- "add" event handler
		local v = getVec4(rhs)
		return setmetatable({x = self.x + v.x, y = self.y + v.y, z = self.z + v.z, w = self.w + v.w}, mt)
  end,
	__sub = function (self, rhs) -- "sub" event handler
		local v = getVec4(rhs)
		return setmetatable({x = self.x - rhs, y = self.y - rhs, z = self.z - rhs, w = self.w - v.w}, mt)
  end,
	__mul = function (self, rhs) -- "mul" event handler
		local v = getVec4(rhs)
		return setmetatable({x = self.x * rhs, y = self.y * rhs, z = self.z * rhs, w = self.w * v.w}, mt)
	end,
	__div = function (self, rhs) -- "div" event handler
		local v = getVec4(rhs)
		return setmetatable({x = self.x / rhs, y = self.y / rhs, z = self.z / rhs, w = self.w / v.w}, mt)
	end,
	__index = {
		toParams = function(self)
			return self.x, self.y, self.z, self.w
		end,
		toArray = function(self)
			return arr4.new(self)
		end,
		toString = function(self)
			return tostring(self)
		end,
	},
	__tostring = function(self)
		return string.format("{x = %.4f, y = %.4f, z = %.4f, w = %.4f}", self:toParams())
	end,
}
------------------------------------------------------------------------------------
--	functionClass.
------------------------------------------------------------------------------------
functionClass = {
	------------------------------------------------------------------------------------
	--	vec4.new(x, y, z, w):
	--	1.	x is a number. Then y, z and w can also be numbers. If not, they take the value of x.
	--	2.	x is a point (arr3)
	--	3.	x is an arr2, then z becomes 0
	--	4.	x is a vec3 / vec4
	--	5.	x is an vec2, then z becomes 0
	--	Otherwise returns vec3 with {x = 0, y = 0, z = 0}
	------------------------------------------------------------------------------------
	new = function(x, y, z, w)
		return setmetatable(getVec4(x, y, z, w), mt)
	end,
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
local modulDB = setmetatable({}, {
	__index = function(self, key)
		if modules then
			for modul in modules:_pairs() do
				local value = modul.handler[key]
				if (value ~= nil) then
					return value
				end
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})

local vec4 = require "vec4"
return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, functionClass, modulDB, vec4}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
