--- tabledata_handler
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.module.tabledata_handler

local historie = [[
	Version 1.1:
	 - Release
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 1,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback

local table
------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		table = global.table
	end,
}

------------------------------------------------------------------------------------
--	meta-class
------------------------------------------------------------------------------------
local metaClass
metaClass = {
	-------------------------------------------------------------------
	--	Tabelle metaData
	--	
	-------------------------------------------------------------------
	metaData = {
		name = "tabledata_handler",
		library = "modutils",
		priority = 5,
	},
	-------------------------------------------------------------------
	--	init
	--	
	-------------------------------------------------------------------
	init = function()
	end,
	-------------------------------------------------------------------
	--	Tabelle handler
	--	
	-------------------------------------------------------------------
	handler = {
		-------------------------------------------------------------------
		--	createUI
		--	
		-------------------------------------------------------------------
		createUI = function(params)
			local handleTableData = table.openKey(params, "userSelection.handleTableData", false, true)
			if (type(handleTableData) == "table") then
				for entry in handleTableData:_pairs() do
					-------------------------------------------------------------------
					--	create UI
					-------------------------------------------------------------------
					if (entry.paramsEntry.uiType == "BUTTON") then
						params.myParams.createButton(entry.paramsEntry)
					elseif (entry.paramsEntry.uiType == "SLIDER") then
						params.myParams.createSlider(entry.paramsEntry)
					else
						params.myParams.createCombobox(entry.paramsEntry)
					end
				end
			end
		end,
		-------------------------------------------------------------------
		--	run
		--	
		-------------------------------------------------------------------
		run = function(params)
			local handleTableData = table.openKey(params, "userSelection.handleTableData", false, true)
			if (type(handleTableData) == "table") then
				local userParams = params.modutilsIntern.userParams
				for entry in handleTableData:_pairs() do
					local uSelection = userParams[entry.paramsEntry.key] or 1
					if (uSelection > 1) then
					
						global.mod.addModifier("loadModel",
							function(fileName, data)
								for tableData in eatSetContainer(entry.data):_pairs() do
									if eatSetContainer(tableData.files):contains(fileName, function(val, fileName) return string.ends(fileName, val) end) then
										for baseKey, keyData in pairs(tableData.data[uSelection - 1]) do
											local baseKeyTable = table.openKey(data, baseKey)
											if (type(baseKeyTable) == "table") then
												for key, _data in pairs(keyData) do
													baseKeyTable[key] = _data
												end
											end
										end
									end
								end
								
								return data
							end,
							metaClass.metaData.priority
						)
						
					end
				end
			end
		end,
	},
}

local mt = {
	__index = function(self, key)
		for i, class in ipairs({libResult, metaClass}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
}

return setmetatable({}, mt)