--- fileshandler
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.module.fileshandler

local historie = [[
	Version 1.1:
	 - Release
	Version 1.2:
	 - Changed: openKey. Neuer Parameter "createIfNotExists". Wenn "true", werden Untertabellen erstellt, falls nicht vorhanden.
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 2,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback

local string
------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		string = global.string
	end,
}

------------------------------------------------------------------------------------
--	meta-class
------------------------------------------------------------------------------------
local metaClass

metaClass = {
	------------------------------------------------------------------------------------
	--	Tabelle metaData
	--	
	------------------------------------------------------------------------------------
	metaData = {
		name = "Table Extend 01",
		library = "table",
	},
	------------------------------------------------------------------------------------
	--	init
	--	
	------------------------------------------------------------------------------------
	init = function()
	end,
	------------------------------------------------------------------------------------
	--	Tabelle handler
	--	
	------------------------------------------------------------------------------------
	handler = {
		------------------------------------------------------------------------------------
		--	proxyTable
		--	
		------------------------------------------------------------------------------------
		proxyTable = function(master)
			return setmetatable({}, 
				{
					__index = master,
					__newindex = master,
					__len = function(self)
						return #master
					end,
					__pairs = function(self)
						return function(self, key)
							local value
							key, value = next(master, key)
							if (value ~= nil) then
								return key, value
							end
						end, self, nil
					end,
					__ipairs = function(self)
						return function(self, i)
							i = i + 1
							local value = master[i]
							if (value ~= nil) then
								return i, value
							end
						end, self, 0
					end,
				}
			)
		end,
		------------------------------------------------------------------------------------
		--	openKey
		--	
		--	Öffnet Untertabelle "key" von "tbl".
		--	Ist "createIfNotExists == true) werden nicht existierende Untertabellen
		--	erstellt; Andernfalls Rückgabe "nil".
		--	Aufruf (Beispiel): table.openKey(metadata, "railVehicle.configs"), true)
		--		Öffnet "railVehicle.configs" unter "metadata" und erstellt
		--		"railVehicle" und auch "configs", fall nicht vorhanden.
		--	Ist resultAsContainer == true dann Rückgabe als eatSetContainer.
		------------------------------------------------------------------------------------
		openKey = function(tbl, key, createIfNotExists, resultAsContainer)
			local keys = string.split(key, ".")
			local result = tbl
			for _key in keys:_pairs() do
				if (type(result[_key]) ~= "table") then
					if (createIfNotExists == true) then
						result[_key] = {}
						result = result[_key]
					else
						return nil
					end
				else
					result = result[_key]
				end
			end
			return (resultAsContainer == true) and eatSetContainer(result) or result
		end,
	},
}

------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
return setmetatable({}, {
	__index = function(self, key)
		for i, class in ipairs({libResult, metaClass}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
})
