--- yearhandler
-- @author Enno Sylvester
-- @copyright 2023
-- @module eatglobal.module.yearhandler

local historie = [[
	Version 1.1:
	 - Release
	Version 1.2:
	 - Added:	Parameter "order" implementiert.
]]

local versionsData = {
	majorVersion = 1,
	minorVersion = 2,
}

------------------------------------------------------------------------------------
--	Variablen gesetzt in "postLoadFn".
------------------------------------------------------------------------------------
local global
local getVar
local setVar
local logFn
local logFnTraceback

local table
local getValue
------------------------------------------------------------------------------------
--	Vordefinition der Rückgabetabelle.
------------------------------------------------------------------------------------
local libResult

------------------------------------------------------------------------------------
--	Unsere Rückgabetabelle.
------------------------------------------------------------------------------------
libResult = {
	__debug__ = false,
	historie = historie,
	------------------------------------------------------------------------------------
	--	postLoadFn - Automatischer Aufruf durch eatglobal nach Laden der Bibliothek.
	------------------------------------------------------------------------------------
	postLoadFn = function(params)
		-------------------------------------------------------------------
		--	Lokale Variable "global", etc. zuweisen.
		-------------------------------------------------------------------
		global = params.eatglobal
		getVar = params.getVar
		setVar = params.setVar
		logFn = params.logFn
		logFnTraceback = params.logFnTraceback
		libResult.version = tostring(eatVersion.new(versionsData))
		
		table = global.table
		getValue = global.misc.getValue
	end,
}

------------------------------------------------------------------------------------
--	meta-class
------------------------------------------------------------------------------------
local metaClass
metaClass = {
	-------------------------------------------------------------------
	--	Tabelle metaData
	--	
	-------------------------------------------------------------------
	metaData = {
		name = "yearhandler",
		library = "modutils",
		priority = 10,
	},
	-------------------------------------------------------------------
	--	init
	--	
	-------------------------------------------------------------------
	init = function()
	end,
	-------------------------------------------------------------------
	--	Tabelle handler
	--	
	-------------------------------------------------------------------
	handler = {
		-------------------------------------------------------------------
		--	createUI
		--	
		-------------------------------------------------------------------
		createUI = function(params)
			local intern = params.modutilsIntern
			local yearFiles = table.openKey(intern, "moduleData.yearhandler.yearFiles", true, true)
			yearFiles:append(getValue(intern.generalYearFiles, {}))
			yearFiles:append(getValue(intern.availableModelFiles, {}))
			logFn("yearFiles = ", yearFiles)
			if (#yearFiles > 0) then
				local userSelection = global.misc.getValue(params.userSelection, {})
				
				if (type(userSelection.disableStartYear) == "table") then
					local entry = userSelection.disableStartYear
					entry.defaultIndex = (entry.default == false) and 1 or 0
					params.myParams.createButton(entry)
				end
				if (type(userSelection.disableEndYear) == "table") then
					local entry = userSelection.disableEndYear
					entry.defaultIndex = (entry.default == false) and 1 or 0
					params.myParams.createButton(entry)
				end
			end
		end,
		-------------------------------------------------------------------
		--	run
		--	
		-------------------------------------------------------------------
		run = function(params)
			-------------------------------------------------------------------
			--	Tools
			-------------------------------------------------------------------
			local function doCompare(val, value)
				return string.ends(value, val)
			end
			local function doChangeData(key, yearData)
				if yearData.handleStartYear then
					key.yearFrom = 0
				end
				if yearData.handleEndYear then
					key.yearTo = 0
				end
			end
			-------------------------------------------------------------------
			--	HIER geht's los!!!! :)
			-------------------------------------------------------------------
			local userSelection = getValue(params.userSelection, {})
			if ((type(userSelection.disableStartYear) == "table") or (type(userSelection.disableEndYear) == "table")) then
				local intern = params.modutilsIntern
				local yearFiles = table.openKey(intern, "moduleData.yearhandler.yearFiles", true, true)
				
				if (#yearFiles > 0) then
					local yearData = {
						handleStartYear = intern.userParams[userSelection.disableStartYear.key] == 1,
						handleEndYear = intern.userParams[userSelection.disableEndYear.key] == 1,
					}
					global.mod.addModifier("all", 
						function(fileName, data, key)
							if (yearFiles:contains(fileName, doCompare)) then
								logFn("yearHandler. run on ", fileName)
								local subKey = global.mod.getSubKey(key)
								if (subKey ~= "loadSoundSet") then
									if (subKey == "loadModel") then
										if ((type(data.metadata) == "table") and (type(data.metadata.availability) == "table")) then
											doChangeData(data.metadata.availability, yearData)
										end
									elseif (subKey == "loadConstruction") then
										if ((type(data.availability) == "table")) then
											doChangeData(data.availability, yearData)
										end
									else
										doChangeData(data, yearData)
									end
								end
							end
							
							return data
						end,
						metaClass.metaData.priority
					)
				end
			end
		end,
	},
}

local mt = {
	__index = function(self, key)
		for i, class in ipairs({libResult, metaClass}) do
			local value = class[key]
			if (value ~= nil) then
				return value
			end
		end
	end,
	__newindex = function(self, key, value) end,
	__metatable = false,
}

return setmetatable({}, mt)
